// Project Sword & Gun Copyright © 2021 Kevin Poretti


#include "Components/SNGCharacterMovementComponent.h"

#include "Kismet/KismetMathLibrary.h"

USNGCharacterMovementComponent::USNGCharacterMovementComponent()
{
	// Defaults
	DefaultGroundFriction = 8.0f;
	DefaultGravityScale = 1.0f;
	DefaultMaxWalkSpeed = 600.0f;
	DefaultRotationRate = FRotator(0.0f, 360.0f, 0.0f);
	
	// sprint
	SprintingMaxWalkSpeed = 800.0f;
	SprintLateralSpeedThreshold = 300.0f;
	SprintingRotationRate = FRotator(0.0f, 180.0f, 0.0f);
	TimeToInitiateSprint = 4.0f;
	WalkToSprintInterpSpeed = 1.0f;

	// firing
	FiringMaxWalkSpeed = 300.0f;
	
	// reloading
	ReloadingMaxWalkSpeed = 300.0f;
	
	// Set defaults dealing with jumping and being airborne
	DefaultJumpForce = 1000.0f;
	DefaultAirFriction = 150.0f;
	DefaultAirControl = 0.35;
	
	MinJumpHeight = 200.0f;
	MaxJumpTime = 0.2f;

	LateralVelocityDampening = 0.8f;
	
	MinAllowedDashHeight = 175.0f;
	MinAllowedAttackHeight = 175.0f;
}

/**
 *	Instantly sets all movement parameters to their defaults values (i.e. ground friction to DefaultGroundFriction,
 *	MaxWalkSpeed to DefaultMaxWalkSpeed, etc.)
 */
void USNGCharacterMovementComponent::RestoreMovementDefaults()
{
	GroundFriction = DefaultGroundFriction;
	GravityScale = DefaultGravityScale;
	RotationRate = DefaultRotationRate;
	MaxWalkSpeed = DefaultMaxWalkSpeed;
	BrakingDecelerationFalling = DefaultAirFriction;
	AirControl = DefaultAirControl;
	JumpZVelocity = DefaultJumpForce;
}

void USNGCharacterMovementComponent::OnSprintTimer()
{
	bIsSprinting = true;
}

void USNGCharacterMovementComponent::StartJump()
{
	/*
	JumpTimer = MaxJumpTime;

	Velocity = FVector(Velocity.X * LateralVelocityDampening, Velocity.Y * LateralVelocityDampening, 0.0f);
	*/
}

void USNGCharacterMovementComponent::AddJumpForce()
{
	/*
	if(JumpTimer > 0.0f)
	{
		Velocity = FVector(Velocity.X, Velocity.Y, Velocity.Z + (JumpForce * GetWorld()->DeltaTimeSeconds));

		JumpTimer = FMath::Max(JumpTimer - GetWorld()->DeltaTimeSeconds, 0.0f);
	}
	*/
}

void USNGCharacterMovementComponent::BeginPlay()
{
	Super::BeginPlay();

	// set character movement parameters to defaults
	RestoreMovementDefaults();
}

void USNGCharacterMovementComponent::TickComponent(float DeltaTime, ELevelTick Tick,
	FActorComponentTickFunction* ThisTickFunction)
{
	Super::TickComponent(DeltaTime, Tick, ThisTickFunction);

	float LateralSpeed = Velocity.Size2D();
	if(LateralSpeed >= SprintLateralSpeedThreshold && !GetWorld()->GetTimerManager().IsTimerActive(TimerHandle_SprintTimer) )
	{
		GetWorld()->GetTimerManager().SetTimer(TimerHandle_SprintTimer,
			this, &USNGCharacterMovementComponent::OnSprintTimer,
			TimeToInitiateSprint, false);
	}

	if(LateralSpeed < SprintLateralSpeedThreshold && GetWorld()->GetTimerManager().IsTimerActive(TimerHandle_SprintTimer))
	{
		GetWorld()->GetTimerManager().ClearTimer(TimerHandle_SprintTimer);
		bIsSprinting = false;
	}
	
	// set walk speed and rotation rate depending on whether player is sprinting or not
	float TargetMaxWalkSpeed = bIsSprinting ? SprintingMaxWalkSpeed : DefaultMaxWalkSpeed;
	float NewMaxWalkSpeed = FMath::FInterpTo(MaxWalkSpeed, TargetMaxWalkSpeed, DeltaTime, WalkToSprintInterpSpeed);
	MaxWalkSpeed = NewMaxWalkSpeed;

	// NOTE(kevin): since default rotate rate is 360 RInterpTo will go from 0 to 180 instead of 360 to 180
	// need to think of a way to prevent this
	// might just need to do my own interp manually
	/*
	FRotator TargetRotationRate = bIsSprinting ? SprintingRotationRate : DefaultRotationRate;
	FRotator NewRotationRate = FMath::RInterpTo(RotationRate, TargetRotationRate, DeltaTime, WalkToSprintInterpSpeed);
	RotationRate = NewRotationRate;
	*/
}
